
                   
define([
    'sna/model/ContextModel',
    'sna/util/ContextPreferenceUtil',
    'dojo/Deferred',
    'dojo/_base/declare',
    'sna/model/AdvancedSearchModel',
    'sna/data/AdvancedSearchData'
], function(ContextModel, ContextPreferenceUtil, Deferred, declare, AdvancedSearchModel, advancedSearchData) {
	var AdvancedSearchController = declare([], {
		constructor: function(args) {
			this.viewSwitcherController = args.viewSwitcherController;
			
			this.model = new AdvancedSearchModel({
            	controller: this
            });

            var _this = this;
            ContextModel.watch('advancedSearchAvailable', function(name, oldValue, newValue) {
                if(newValue === true) {
                    _this.enable();
                }
                else {
                    _this.disable();
                }
            });
		},
		
        close: function() {
            this.model.close();
        },

        disable: function() {
        	this.model.disableButton();
        },

        enable: function() {
        	this.model.enableButton();
        },

        start: function( vertical, uiElements, hideCloseButton ) {
        	if( !hideCloseButton ) {
        		hideCloseButton = false;
        	}
        	
            this.model.render(vertical, uiElements, hideCloseButton); 
            this.model.setUpPreferences();
        },
        
        loadAndStartAdvancedSearch: function(vertical, hideCloseButton, bypassMode) {
            var deferred = new Deferred();

        	var that = this; 
            advancedSearchData.getAdvancedSearchUI(vertical, function(data) {
                console.log('AdvancedSearchController.loadAndStartAdvancedSearch', data);

                ContextModel.set('advancedSearchAvailable', data.hasAdvancedSearch);
                if(data.hasAdvancedSearch) {
                    var bypass = ContextPreferenceUtil.getPreference("bypass_initial_search", vertical) === 'y';
                    deferred.resolve({
                        hasAdvancedSearch: true,
                        bypass: bypass
                    });

                    if(bypass && (bypassMode == AdvancedSearchController.BYPASS_MODE.BYPASS_ON_PREFERENCE || bypassMode == AdvancedSearchController.BYPASS_MODE.BYPASS_ALWAYS)) {
                        
                    }
                    else if (bypassMode == AdvancedSearchController.BYPASS_MODE.BYPASS_ALWAYS) {
                        
                    }
                    else {
                        that.start(vertical, data, hideCloseButton);
                    }
                }
                else {
                    deferred.resolve({
                        hasAdvancedSearch: false,
                        bypass: false
                    });
                }
            });

            return deferred;
        },

        createRowCounts: function(min, max, step) {
            var values = [];
            var i = 0;

            for(i = min; i < max; i += step) {
                values.push({
                    label: i,
                    value: i
                });
            }

            return values;
        },

        submissionFunction: function( event, vertical, submissionsMap, fetchEntities) {
        	if(typeof(fetchEntities) === 'undefined') {
        		console.error('AdvancedSearchController.submissionFunction: fetchEntities was undefined');
        	}
        	
            this.close();
            this.viewSwitcherController.switchToAlertsListPage(true, vertical, submissionsMap, fetchEntities, undefined);
        } 
    });

    AdvancedSearchController.BYPASS_MODE = {
        'NONE': 0,
        'BYPASS_ALWAYS': 1,
        'BYPASS_ON_PREFERENCE': 2
    };

    return AdvancedSearchController;
}); 
               