
                   
define([
    'dojo/_base/declare',
    '../model/DispositionModel',
    '../data/DispositionData',
    'dojox/lang/functional'
], function(declare, DispositionModel, dispositionData, functional) {
    var POSSIBLE_DIALOG_ACTIONS = {
        NOOP: 0,
        UPDATE_AE: 1,
        REMOVE_AE: 2,
        FULL_REFRESH: 3
    };

    return declare([], {
        constructor:function (args) {
            this.alertDetailsController = args.alertDetailsController;
            this.alertsViewController = args.alertsViewController;
            this.viewSwitcherController = args.viewSwitcherController;
        },

        start:function (vertical, callback, fromDirectLink) {
            var that = this;

            dispositionData.getDispositionActions(vertical, 'sasdemo', function(data) {
                console.log('disposition data', data);
                
                that.model = new DispositionModel({
                    dispositionElements: data,
                    vertical: vertical,
                    controller: that
                });

                that.model.setFromDirectLink(fromDirectLink);

                that.viewable = data && data.length > 0;

                
                
                if( data && !fromDirectLink) {
                	that.enable();
                }
                else {
                	that.disable();
                }

                if( callback ) {
                    callback();
                }
            });
        },

        
        updateDispositionItemsList: function(selectedEntitiesArray, currentView) {
            if(typeof(selectedEntitiesArray[0]) === 'undefined') {
                selectedEntitiesArray.splice(0, 1);
            }

            for (var i = 1; i < selectedEntitiesArray.length; i += 1) {
                if(typeof(selectedEntitiesArray[i]) === 'undefined' || (typeof(selectedEntitiesArray[i-1]) !== 'undefined' && selectedEntitiesArray[i].actionableEntityId === selectedEntitiesArray[i-1].actionableEntityId)) {
                    selectedEntitiesArray.splice(i, 1);
                    i--;
                }
            }

            if(this.model) {
                this.model.updateDispositionList(selectedEntitiesArray, currentView);
            }

            if (!this.viewable) {
                this.disable();
            }
        },

        
        filterDispositionItemsByView: function(currentView, dispositionElementsMap) {
            var results = [];

            functional.forIn(dispositionElementsMap, function(value, key) {
                var regex;
                if(value.view !== '*') {
                    regex = new RegExp(value.view, 'i');
                }

                var entryMatches = false;

                if(value.view === '*' || regex.test(currentView)) {
                    entryMatches = true;
                }

                if(entryMatches) {
                    results.push(value);
                }
            });

            return results;
        },

        
        filterDispositionItemArray: function(selectedEntitiesArray, dispositionElementsMap) {
            var results = [];

            functional.forIn(dispositionElementsMap, function(value, key) {
                var regex;
                if(value.entityType !== '*') {
                    regex = new RegExp(value.entityType, 'i');
                }

                var entryMatches = false;

                if(value.entityType === '*' ||  selectedEntitiesArray.length === 0 || regex.test(selectedEntitiesArray[0].actionableEntityType)) {
                    entryMatches = true;
                }

                if(!value.supportsMultiple && selectedEntitiesArray.length > 1) {
                    entryMatches = false;
                }

                if(selectedEntitiesArray.length === 0) {
                    entryMatches = false;
                }

                results.push({
                    value: value,
                    enabled: entryMatches
                });
            });

            return results;
        },

        dispositionActionClicked: function(vertical, dispositionKey, parametersMap) {
        	var that = this; 
            console.log('dispositioncontroller.dispositionActionClicked');

            dispositionData.getDispositionUI(vertical, dispositionKey, parametersMap, function(uiDataMap) {
                that.model.renderDispositionUI(uiDataMap, dispositionKey);
            });
        },

        populateParametersMap: function(parametersArray, selectedEntitiesArray) {
            var parametersMap = {};

            functional.forIn(parametersArray, function(value, key) {
                if(value.startsWith('entity.')) {
                    parametersMap[value] = selectedEntitiesArray[0][value.replace('entity.', '')];
                }
            });

            return parametersMap;
        },

        
        submitDispositionResponse: function(vertical, actionableEntitiesArray, dispositionCriteria, selectedPanel, callback) {
            console.log("DispositionController.submitDispositionResponse", vertical, actionableEntitiesArray, dispositionCriteria, selectedPanel, callback);

            dispositionData.submitAction(vertical, selectedPanel, actionableEntitiesArray, dispositionCriteria, function(data) {
                if(callback) {
                    callback(data);
                }
            });
        },

        
        takeActionBasedOnResponse:  function(code, isDetailsView, selectedEntities, vertical) {
            console.log('DispositionController.takeActionBasedOnResonse', code, isDetailsView, selectedEntities, vertical);

            if(code === POSSIBLE_DIALOG_ACTIONS.REMOVE_AE) {
                if(!isDetailsView) {
                    this.alertsViewController.removeCheckedRows();
                }
                else {
                    this.viewSwitcherController.switchToAlertsListPage(true, vertical, this.alertsViewController.getAdvancedSearchParameters(),false);
                    this.alertsViewController.removeRows(selectedEntities);
                }
            }
            else if (code === POSSIBLE_DIALOG_ACTIONS.FULL_REFRESH) {
                    this.viewSwitcherController.switchToAlertsListPage(true, vertical, this.alertsViewController.getAdvancedSearchParameters(),true, false);
            }
        },

        alertDetailsIsLoaded:function () {
            return this.alertDetailsController.isLoaded()
        },

        dispositionCompleted:function () {
            this.alertsViewController.dispositionCompleted();
        },

        disable:function () {
            if (typeof(this.model) !== "undefined" && this.model !== null) {
                this.model.disableButton();
            }
        },

        enable:function () {
            if (this.viewable) {
                this.model.enableButton();
            }
        }
    });
});

               