
                   define([
    'dojo/when',
    'dojo/Deferred',
    'dojox/lang/functional',
    'dojo/dom-style',
    'dojo/dom-geometry',
    'dojo/_base/array',
    'dijit/layout/TabContainer',
    'dijit/Dialog',
    'dijit/form/Button',
    'dijit/layout/ContentPane',
    'dijit/registry',
    'dojo/_base/declare',
    'dojo/dom',
    'dojo/dom-construct',
    'dojo/i18n!./nls/validate',
    'dojo/on',
    'dojo/query',
    'dojo/request/xhr',
    'sna/public/api/util/IconUtil',
    'dijit/CheckedMenuItem'
], function(when, Deferred, functional, domStyle, domGeometry, array, TabContainer, Dialog, Button, ContentPane, registry, declare, dom, domConstruct, validate, on, query, xhr, IconUtil, CheckedMenuItem) {
    return declare([], {

        constructor: function() {
            this._networkIcons = new Deferred();
        },

        _saveDialogPosition: function(){
            if(this._dialog && this._dialog.domNode){
                this.left = domStyle.get(this._dialog.domNode,"left");
                this.top = domStyle.get(this._dialog.domNode,"top");
            }
        },

        _restoreDialogPosition: function(){
            if(this._dialog && this._dialog.domNode){
                domStyle.set(this._dialog.domNode, {
                    left: this.left + 'px',
                    top: this.top + 'px'
                });
            }
        },

        
        _positionDialog: function(networkView) {
            if(this.left){
                this._restoreDialogPosition();
            }
            else{
                
                var diagramPane = query('.networkdiagram', networkView.domNode)[0];
                var position = domGeometry.position(diagramPane);
                var dimensions = domGeometry.position(this._dialog.domNode);

                domStyle.set(this._dialog.domNode, {
                    left: position.x + position.w - dimensions.w - 15 + 'px',
                    top: position.y + position.h - dimensions.h - 15 + 'px'
                });
            }
        },

        
        _setupToolbar: function(networkView, data, vertical) {
            
            var previousNodes = query('.networklegendbutton');
            if(previousNodes.length > 0) {
                registry.byNode(previousNodes[0]).destroyRecursive();
            }

            
            var previousDialog = query('.networklegenddialog');
            
            
            if(previousDialog.length > 0) {
                registry.byNode(previousDialog[0]).destroyRecursive();
            }

            
            var that = this;
            var button = new CheckedMenuItem({
                label: validate['sna.plugins.networklegend.title'],
                "class": "networklegendbutton",
                "data-dojo-attach-point": 'networklegendbutton',
                showLabel: true
            });
            that.button = button;
            networkView.own(on(button, 'click', function() {
                if (that._dialog && that._dialog.get('open')) {
                    that._saveDialogPosition();
                    that._dialog.destroyRecursive();
                    that._dialog = undefined;
                }
                else {
                    that._legendButtonClicked(networkView, data, vertical);
                }
            }));

            networkView.showdropdown.addChild(button);

            
            
            networkView.own(on(networkView.getParent(), 'hide', function() {
                if(that._dialog) {
                    
                    
                    
                    if(that._dialog._destroyed){
                        that._dialog = undefined;
                    }else{
                        that._dialog.hide();
                        that._saveDialogPosition();
                    }
                }
            }));

            
            
            
            networkView.own(on(networkView.getParent(), 'show', function() {
                if(that._dialog && !that._dialog._destroyed && button && button.get('checked')) {
                    that._dialog.show();
                    that._restoreDialogPosition();
                }
            }));

            
            networkView.own({
                remove: function() {
                    if(that._dialog) {
                        that._dialog.destroyRecursive();
                    }
                }
            });
        },

        _createColorModelEntry: function(item) {

            var label = '';
            if(item.indicatorName.trim().length > 0) {
                label = item.indicatorName + ': ';
            }
            
            function isNumber (o) {
                return ! isNaN (o-0) && o !== null && o.replace(/^\s\s*/, '') !== "" && o !== false;
            }

            var lowerInclusive = '';
            var upperInclusive = '';

            if(isNumber(item.lowerBound)) {
                if(item.lowerBoundInclusive) {
                    lowerInclusive = '=';
                }
                if(item.upperBoundInclusive) {
                    upperInclusive = '=';
                }

                label += ">%0%1 - <%2%3".tokenize(lowerInclusive, item.lowerBound.trim(), upperInclusive, item.upperBound.trim());
            }
            else {
                label += item.lowerBound;
            }

            return label;
        },

        
        execute: function(data, vertical) {
            var _this = this;
            IconUtil.updateIconArrayCache(data.component.bundlePrefix, vertical, function(iconArray){
                _this._networkIcons.resolve(iconArray);
                _this._networkIcons = iconArray;
            });

            this._setupToolbar(data.component, data, vertical);
        },

        
        _legendButtonClicked: function(networkView, data, vertical) {
            var that = this;
            
            var imagesLoc='src/sna/resources/images/network/';
            var ae_type = data.networkState.actionableEntityType;

            if(this._dialog) {
                this._dialog.destroyRecursive();
            }

            var collectNodeIcons = function() {
                var icons = {};
                array.forEach(networkView.networkState.nodes, function(node) {
                    functional.forIn(node.evolutions2, function(evolution) {
                        if(evolution.nodeSymbol) {
                            icons[evolution.nodeSymbol] = true;
                        }
                    });
                });

                return Object.keys(icons).sort();
            };

            function buildLegend(context){
                if(!context.legendMetaIcons) {
                    context.legendMetaIcons = [];
                    context.error = true;
                }

                that._contextIcons=context.legendMetaIcons;

                var nodeSymbols = collectNodeIcons();
                when(that._networkIcons, function(iconArray) {
                    that._nodePNGs=[];
                    that._nodeLegendLabels=[];

                    for(var x=0; x<nodeSymbols.length; x++){
                        for(var z=0; z<iconArray.length; z++){
                            for(var y=0; y<iconArray[z].values.length; y++){
                                if (iconArray[z].values[y]==nodeSymbols[x]){
                                    that._nodePNGs[x]=iconArray[z].filename;
                                    that._nodeLegendLabels[x]=iconArray[z].label;
                                    break;
                                }
                            }
                            if (typeof that._nodePNGs[x]!='undefined') { break; }
                        }
                    }

                    var dialog = new Dialog({
                        "class": "networklegenddialog",
                        "title": validate['sna.plugins.networklegend.title'],
                        isNonModal: true
                    });

                    networkView.own(on(dialog, 'cancel', function() {
                        that._saveDialogPosition();
                        that.button.set('checked',false);
                    }));

                    var tabContainer = new TabContainer({
                        style: 'width:300px; height: 300px;'
                    });

                    dialog.startup();
                    domConstruct.place(tabContainer.domNode, dialog.containerNode);
                    tabContainer.startup();

                    var iconPane = new ContentPane({
                        title: validate['sna.plugins.networklegend.icons']
                    });

                    var colorPane = new ContentPane({
                        title: validate['sna.plugins.networklegend.colors'],
                        style: "padding-bottom: 5px;"
                    });

                    tabContainer.addChild(iconPane);
                    tabContainer.addChild(colorPane);

                    that._dialog = dialog;

                    var propPanel = iconPane.domNode;

                    
                    if (!data.component.legendCreated){
                        var cpLegend = new ContentPane({region: 'center', style: 'height: %80; width: 100%;'});
                        data.component.legendCreated=true;
                        data.component.cpLegend=registry.byId(cpLegend);
                    }
                    else {
                        cpLegend=data.component.cpLegend;
                        cpLegend.destroyRendering(false);
                        cpLegend.destroyRecursive(true);
                        var cpLegend = new ContentPane({region: 'center', style: 'height: %80; width: 100%;'});
                        data.component.cpLegend=registry.byId(cpLegend);
                    }

                    cpLegend.startup();
                    domConstruct.place(cpLegend.domNode, propPanel);
                    dialog.show();
                    that._positionDialog(networkView);

                    domConstruct.destroy(query('.networklegenddialog_underlay')[0]);

                    var cpLegendId=registry.byId(cpLegend).id;

                    var wholelegendid=cpLegendId+'wl';

                    domConstruct.create('div', {id : wholelegendid}, cpLegendId, 'last');
                    query('#'+wholelegendid).style('margin','7px 7px 10px 7px').style('padding','0px');
                    var wholeLegend = dom.byId(wholelegendid);

                    var legendlistid=cpLegendId+'ll'
                    domConstruct.create('ul', {id : legendlistid}, wholeLegend, 'last');
                    query('#'+legendlistid).style('margin','0px 0px 0px 0px').style('padding','0px').style("listStyle", "none outside none");
                    var legendlist=dom.byId(legendlistid);

                    var legenditems=cpLegendId+'li',
                        legendicons=cpLegendId+'lic';
                    for(var w=0; w<nodeSymbols.length; w++){
                        domConstruct.create('li', {innerHTML : '<div class="'+legenditems+'"><img class="'+legendicons+'" src="'+imagesLoc+that._nodePNGs[w]+'" width="28" height="28"/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'+that._nodeLegendLabels[w]+'</div>' }, legendlist, 'last');
                    }

                    if(nodeSymbols.length === 0 && !context.error) {
                        domConstruct.create('li', {innerHTML: validate["sna.plugins.networklegend.noicons"]}, legendlist);
                    }
                    else if (nodeSymbols.length === 0 && context.error) {
                        domConstruct.create('li', {innerHTML: validate["sna.plugins.networklegend.corrupticons"]}, legendlist);
                    }

                    var colorModelList = domConstruct.create('ul', {
                        style: 'margin: 0; padding: 0; list-style: none; height: 100%; overflow: auto; margin: 0px 0px 0px 7px;'
                    }, colorPane.domNode);

                    function createColorIcon(color,label,number) {
                        var canvasid=cpLegendId+"_cL_"+number;

                        domConstruct.create('li', {innerHTML : '<div class="'+legenditems+'"><canvas class="'+legendicons+'" id="'+canvasid+'" width="28" height="28"></canvas>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'+label+'</div>' }, colorModelList, 'last');
                        var thisone = dom.byId(canvasid).getContext('2d');

                        thisone.beginPath();
                        thisone.rect(1,1,28,28);
                        thisone.fillStyle=color;
                        thisone.fill();
                        thisone.lineWidth=1;
                        thisone.strokeStyle='#c0c0c0';
                        thisone.stroke();
                    }

                    if(data.component.networkState.colorModel && data.component.networkState.colorModel.length > 0) {
                        array.forEach(data.component.networkState.colorModel, function(colorModelEntry, index) {
                            var label = that._createColorModelEntry(colorModelEntry);
                            createColorIcon(colorModelEntry.nodeColor.replace("0x", "#"), label, index);
                        });
                    }
                    else {
                        domConstruct.create('li', { innerHTML: validate["sna.plugins.networklegend.nocolors"]}, colorModelList);
                    }

                    if(nodeSymbols.length === 0 && !(data.component.networkState.colorModel && data.component.networkState.colorModel.length > 0)) {
                        tabContainer.destroyRecursive();
                        var errorPane = new ContentPane({
                            style: 'width:300px; height: 300px; padding: 5px;',
                            content: validate["sna.plugins.networklegend.nolegend"]
                        });

                        domConstruct.place(errorPane.domNode, dialog.containerNode);
                        errorPane.startup();
                    }

                    query("."+legenditems).style("margin","0px 0px 5px 0px").style("padding","0px");
                    query("."+legendicons).style("verticalAlign","middle").style("margin","0px 0px 0px 0px").style("padding","0px");
                });

            }

            buildLegend({
                legendMetaIcons: []
            });
        }
    });
});

               